@@ -16,6 +16,7 @@ from glob import glob
 import nox
 
 CI = os.environ.get('CI') is not None
+INSTALL_SDK_FROM = os.environ.get('INSTALL_SDK_FROM')
 NOX_PYTHONS = os.environ.get('NOX_PYTHONS')
 
 PYTHON_VERSIONS = ['3.5', '3.6', '3.7', '3.8', '3.9'
@@ -29,6 +30,9 @@ REQUIREMENTS_LINT = ['yapf==0.27', 'pyflakes==2.2.0', 'pytest==6.1.1', 'liccheck
 REQUIREMENTS_TEST = ['pytest==6.1.1', 'pytest-cov==2.10.1']
 REQUIREMENTS_BUILD = ['setuptools>=20.2']
 
+OSX_BUNDLE_IDENTIFIER = 'com.backblaze.b2'
+OSX_BUNDLE_ENTITLEMENTS = 'contrib/macos/entitlements.plist'
+
 nox.options.reuse_existing_virtualenvs = True
 nox.options.sessions = [
     'lint',
@@ -43,7 +47,8 @@ if CI:
 def install_myself(session, extras=None):
     """Install from the source."""
     # In CI, install B2 SDK from the master branch
-    if CI:
+
+    if CI and not INSTALL_SDK_FROM:
         session.install('git+git://github.com/Backblaze/b2-sdk-python#egg=b2sdk')
 
     arg = '.'
@@ -51,6 +56,12 @@ def install_myself(session, extras=None):
         arg += '[%s]' % ','.join(extras)
 
     session.install('-e', arg)
+    if INSTALL_SDK_FROM:
+        cwd = os.getcwd()
+        os.chdir(INSTALL_SDK_FROM)
+        session.run('pip', 'uninstall', 'b2sdk', '-y')
+        session.run('python', 'setup.py', 'develop')
+        os.chdir(cwd)
 
 
 @nox.session(name='format', python=PYTHON_DEFAULT_VERSION)
@@ -169,6 +180,12 @@ def bundle(session):
     session.install('pyinstaller')
     session.run('rm', '-rf', 'build', 'dist', 'b2.egg-info', external=True)
     install_myself(session)
+
+    system = platform.system().lower()
+
+    if system == 'darwin':
+        session.posargs.extend(['--osx-bundle-identifier', OSX_BUNDLE_IDENTIFIER])
+
     session.run('pyinstaller', '--onefile', *session.posargs, 'b2.spec')
 
     # Set outputs for GitHub Actions
@@ -177,11 +194,38 @@ def bundle(session):
         print('::set-output name=asset_path::', asset_path, sep='')
 
         name, ext = os.path.splitext(os.path.basename(asset_path))
-        system = platform.system().lower()
         asset_name = '{}-{}{}'.format(name, system, ext)
         print('::set-output name=asset_name::', asset_name, sep='')
 
 
+@nox.session(python=False)
+def sign(session):
+    """Sign the bundled distribution (OSX only)."""
+    system = platform.system().lower()
+
+    if system != 'darwin':
+        session.skip('signing process is for OSX only')
+
+    session.run('security', 'find-identity', external=True)
+    session.run(
+        'codesign',
+        '--deep',
+        '--force',
+        '--verbose',
+        '--timestamp',
+        '--identifier',
+        OSX_BUNDLE_IDENTIFIER,
+        '--entitlements',
+        OSX_BUNDLE_ENTITLEMENTS,
+        '--options',
+        'runtime',
+        *session.posargs,
+        'dist/b2',
+        external=True
+    )
+    session.run('codesign', '--verify', '--verbose', 'dist/b2', external=True)
+
+
 @nox.session(python=PYTHON_DEFAULT_VERSION)
 def doc(session):
     """Build the documentation."""
